# This model is based on the PowerFactory-version of the IEEE39-bus system. The following changes were made:
# - G10 is modeled as a salient pole machine in the PowerFactory-model. This is modeled bu setting X_q  = X_q_t, and choosing an arbitrary value for T_q0_t (does not matter).
# - The AVR, GOV and PSS-models are changed to SEXS, TGOV1 and PSTAB1, respectively (since these are the only ones implemented at the moment).


def load():
    return {
        'base_mva': 700,
        'f': 60,
        'slack_bus': '31',

        'buses': [
            ['name',    'V_n'],
            ['1', 345],
            ['2', 345],
            ['3', 345],
            ['4', 345],
            ['5', 345],
            ['6', 345],
            ['7', 345],
            ['8', 345],
            ['9', 345],
            ['10', 345],
            ['11', 345],
            ['12', 138],
            ['13', 345],
            ['14', 345],
            ['15', 345],
            ['16', 345],
            ['17', 345],
            ['18', 345],
            ['19', 345],
            ['20', 230],
            ['21', 345],
            ['22', 345],
            ['23', 345],
            ['24', 345],
            ['25', 345],
            ['26', 345],
            ['27', 345],
            ['28', 345],
            ['29', 345],
            ['30', 16.5],
            ['31', 16.5],
            ['32', 16.5],
            ['33', 16.5],
            ['34', 16.5],
            ['35', 16.5],
            ['36', 16.5],
            ['37', 16.5],
            ['38', 16.5],
            ['39', 345],
        ],

        'lines': [
            ['name',    'from_bus', 'to_bus',   'length',   'V_n',  'unit',     'R',            'X',    'B'],
            ['L1-2',    '1',        '2',        163.0643,   345,    'pf',       0.02554745,     0.3,    3.599926],
            ['L1-39',   '1',        '39',       99.1875,    345,    'pf',       0.012,          0.3,    6.352828],
            ['L2-3',    '2',        '3',        59.90925,   345,    'pf',       0.02582781,     0.3,    3.606938],
            ['L2-25',   '2',        '25',       34.1205,    345,    'pf',       0.244186,       0.3,    3.594987],
            ['L3-4',    '3',        '4',        84.50775,   345,    'pf',       0.01830986,     0.3,    2.2011],
            ['L3-18',   '3',        '18',       52.76775,   345,    'pf',       0.02481203,     0.3,    3.404079],
            ['L4-5',    '4',        '5',        50.784,     345,    'pf',       0.01875,        0.3,    2.220176],
            ['L4-14',   '4',        '14',       51.18075,   345,    'pf',       0.01860465,     0.3,    2.268619],
            ['L5-6',    '5',        '6',        10.3155,    345,    'pf',       0.02307692,     0.3,    3.534782],
            ['L5-8',    '5',        '8',        44.436,     345,    'pf',       0.02142857,     0.3,    2.790714],
            ['L6-7',    '6',        '7',        36.501,     345,    'pf',       0.01956522,     0.3,    2.600975],
            ['L6-11',   '6',        '11',       32.5335,    345,    'pf',       0.02560976,     0.3,    3.587033],
            ['L7-8',    '7',        '8',        18.2505,    345,    'pf',       0.02608696,     0.3,    3.590727],
            ['L8-9',    '8',        '9',        144.0202,   345,    'pf',       0.01900826,     0.3,    2.219121],
            ['L9-39',   '9',        '39',       99.1875,    345,    'pf',       0.012,          0.3,    10.16451],
            ['L10-11',  '10',       '11',       17.06025,   345,    'pf',       0.02790698,     0.3,    3.590086],
            ['L10-13',  '10',       '13',       17.06025,   345,    'pf',       0.02790698,     0.3,    3.590086],
            ['L13-14',  '13',       '14',       40.07175,   345,    'pf',       0.02673267,     0.3,    3.612517],
            ['L14-15',  '14',       '15',       86.09475,   345,    'pf',       0.02488479,     0.3,    3.571614],
            ['L15-16',  '15',       '16',       37.2945,    345,    'pf',       0.0287234,      0.3,    3.852246],
            ['L16-17',  '16',       '17',       35.31075,   345,    'pf',       0.02359551,     0.3,    3.193077],
            ['L16-19',  '16',       '19',       77.36625,   345,    'pf',       0.02461538,     0.3,    3.301274],
            ['L16-21',  '16',       '21',       53.56125,   345,    'pf',       0.01777778,     0.3,    3.996784],
            ['L16-24',  '16',       '24',       23.40825,   345,    'pf',       0.01525424,     0.3,    2.44064],
            ['L17-18',  '17',       '18',       32.5335,    345,    'pf',       0.02560976,     0.3,    3.406228],
            ['L17-27',  '17',       '27',       68.63775,   345,    'pf',       0.02254335,     0.3,    3.936541],
            ['L21-22',  '21',       '22',       55.545,     345,    'pf',       0.01714286,     0.3,    3.879766],
            ['L22-23',  '22',       '23',       38.088,     345,    'pf',       0.01875,        0.3,    4.071994],
            ['L23-24',  '23',       '24',       138.8625,   345,    'pf',       0.01885714,     0.3,    2.184173],
            ['L25-26',  '25',       '26',       128.1503,   345,    'pf',       0.02972136,     0.3,    3.363251],
            ['L26-27',  '26',       '27',       58.32225,   345,    'pf',       0.02857143,     0.3,    3.451542],
            ['L26-28',  '26',       '28',       188.0595,   345,    'pf',       0.02721519,     0.3,    3.485547],
            ['L26-29',  '26',       '29',       247.9688,   345,    'pf',       0.02736,        0.3,    3.486414],
            ['L28-29',  '28',       '29',       59.90925,   345,    'pf',       0.02781457,     0.3,    3.491956],
        ],

        'transformers': [
            ['name',    'from_bus', 'to_bus',    'S_n', 'V_n_from', 'V_n_to', 'N_par',  'R',        'X',        'ratio_from',   'ratio_to'],
            ['T2-30',   '2',        '30',        1000,   345,       16.5,     1,        0,          0.181,      1/1.0250,       1],
            ['T6-31',   '6',        '31',        700,    345,       16.5,     1,        0,          0.175,      1/1.0700,       1],
            ['T10-32',  '10',       '32',        800,    345,       16.5,     1,        0,          0.16,       1/1.0700,       1],
            ['T11-12',  '11',       '12',        300,    345,       138,      1,        0.0048,     0.1304999,  1/1,            1/1.0060],
            ['T13-12',  '13',       '12',        300,    345,       138,      1,        0.0048,     0.1304999,  1/1,            1/1.0060],
            ['T19-20',  '19',       '20',        1000,   345,       230,      1,        0.007,      0.138,      1/1.0600,       1],
            ['T19-33',  '19',       '33',        800,    345,       16.5,     1,        0.0056,     0.1135999,  1/1.0700,       1],
            ['T20-34',  '20',       '34',        300,    230,       16.5,     2,        0.0054,     0.108,      1/1.0090,       1],
            ['T22-35',  '22',       '35',        800,    345,       16.5,     1,        0,          0.1144,     1/1.0250,       1],
            ['T23-36',  '23',       '36',        700,    345,       16.5,     1,        0.0035,     0.1904,     1/1.0000,       1],
            ['T25-37',  '25',       '37',        700,    345,       16.5,     1,        0.0042,     0.1624,     1/1.0250,       1],
            ['T29-38',  '29',       '38',        1000,   345,       16.5,     1,        0.008,      0.156,      1/1.0250,       1],
        ],

        'loads': [
            ['name',    'bus',  'P',    'Q',   'model'],
            ['L3',      '3',    322.,   2.4,   'Z'],
            ['L4',      '4',    500.,   184.,  'Z'],
            ['L7',      '7',    233.8,  84.,   'Z'],
            ['L8',      '8',    522.,   176.,  'Z'],
            ['L12',     '12',   7.5,    88.,   'Z'],
            ['L15',     '15',   320.,   153.,  'Z'],
            ['L16',     '16',   329.,   32.3,  'Z'],
            ['L18',     '18',   158.,   30.,   'Z'],
            ['L20',     '20',   628.,   103.,  'Z'],
            ['L21',     '21',   274.,   115.,  'Z'],
            ['L23',     '23',   247.5,  84.6,  'Z'],
            ['L24',     '24',   308.6,  -92.2, 'Z'],
            ['L25',     '25',   224.,   47.2,  'Z'],
            ['L26',     '26',   139.,   17.,   'Z'],
            ['L27',     '27',   281.,   75.5,  'Z'],
            ['L28',     '28',   206.,   27.6,  'Z'],
            ['L29',     '29',   283.5,  26.9,  'Z'],
            ['L31',     '31',   9.2,    4.6,   'Z'],
            ['L39',     '39',   1104.,  250.,  'Z'],
        ],

        'shunts': [
            ['name',    'bus',  'V_n',  'Q',    'model'],
            # ['C1',      'B7',   230,    200,    'Z'],
            # ['C2',      'B9',   230,    350,    'Z'],
        ],

        'generators': {
            'GEN': [
                ['name',    'bus',     'S_n',  'V_n',   'PF_n', 'P',    'V',     'N_par', 'H',     'D',     'X_d',  'X_q',      'X_d_t',    'X_q_t',    'X_d_st',   'X_q_st',   'T_d0_t',   'T_q0_t',   'T_d0_st',  'T_q0_st'],
                ['G1',      '39',      10000,  345.,    0.85,   1000.,  1.03,    1,       5.,       0,      2.,     1.9,        0.6,        0.8,        0.4,        0.4,        7.,         0.7,        0.05,       0.035],
                ['G2',      '31',      700,    16.5,    0.85,   0.,     0.982,   1,       4.329,    0,      2.065,  1.974,      0.4879,     1.19,       0.35,       0.35,       6.56,       1.5,        0.05,       0.035],
                ['G3',      '32',      800,    16.5,    0.85,   650.,   0.9831,  1,       4.475,    0,      1.996,  1.896,      0.4248,     0.7008,     0.36,       0.36,       5.7,        1.5,        0.05,       0.035],
                ['G4',      '33',      800,    16.5,    0.85,   632.,   0.9972,  1,       3.575,    0,      2.096,  2.064,      0.3488,     1.328,      0.28,       0.28,       5.69,       1.5,        0.05,       0.035],
                ['G5',      '34',      300,    16.5,    0.85,   254.,   1.0123,  2,       4.333,    0,      2.01,   1.86,       0.396,      0.498,      0.267,      0.267,      5.4,        0.44,       0.05,       0.035],
                ['G6',      '35',      800,    16.5,    0.85,   650.,   1.0493,  1,       4.35,     0,      2.032,  1.928,      0.4,        0.6512,     0.32,       0.32,       7.3,        0.4,        0.05,       0.035],
                ['G7',      '36',      700,    16.5,    0.85,   560.,   1.0635,  1,       3.771,    0,      2.065,  2.044,      0.343,      1.302,      0.308,      0.308,      5.66,       1.5,        0.05,       0.035],
                ['G8',      '37',      700,    16.5,    0.85,   540.,   1.0278,  1,       3.471,    0,      2.03,   1.96,       0.399,      0.6377,     0.315,      0.315,      6.7,        0.41,       0.05,       0.035],
                ['G9',      '38',      1000,   16.5,    0.85,   830.,   1.0265,  1,       3.45,     0,      2.106,  2.05,       0.57,       0.587,      0.45,       0.45,       4.79,       1.96,       0.05,       0.035],
                ['G10',     '30',      1000,   16.5,    0.85,   250.,   1.0475,  1,       4.2,      0,      1.,     0.69,       0.31,       0.69,       0.25,       0.25,       10.2,       1,          0.05,       0.035],  # Salient pole generator
            ]
        },

        'gov': {
            'TGOV1': [
                ['name',    'gen',  'R',    'D_t',  'V_min',    'V_max',    'T_1',  'T_2',  'T_3'],
                ['GOV1',     'G1',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV2',     'G2',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV3',     'G3',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV4',     'G4',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV5',     'G5',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV6',     'G6',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV7',     'G7',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV8',     'G8',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV9',     'G9',   0.05,   0,     0,          1,          0.2,    1,      2],
                ['GOV10',    'G10',  0.05,   0,     0,          1,          0.2,    1,      2],
            ]
        },

        'avr': {
            'SEXS': [
                ['name',  'gen',        'K',    'T_a',  'T_b',  'T_e',  'E_min',    'E_max'],
                ['AVR1',    'G1',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR2',    'G2',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR3',    'G3',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR4',    'G4',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR5',    'G5',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR6',    'G6',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR7',    'G7',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR8',    'G8',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR9',    'G9',       100,    2.0,    10.0,   0.5,    -3,         3],
                ['AVR10',   'G10',      100,    2.0,    10.0,   0.5,    -3,         3],
            ]
        },

        'pss': {
            'STAB1': [
                ['name',    'gen',  'K',    'T',    'T_1',  'T_2',  'T_3',  'T_4',  'H_lim'],
                ['PSS1',    'G1',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS2',    'G2',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS3',    'G3',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS4',    'G4',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS5',    'G5',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS6',    'G6',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS7',    'G7',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS8',    'G8',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS9',    'G9',   50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
                ['PSS10',   'G10',  50,     10.0,   0.5,    0.5,    0.05,   0.05,   0.03],
            ]
        },
    }